<?php
/**
*
* @package phpBB3
* @copyright (c) 2008 Justin Aquadro
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/*
* Build an additional index to access badge keys by a group ID.
*
* @param array $badges An array of badges from the cache system.
* @return array An array or arrays of badge IDs, indexed by group ID.
*/
function build_group_badges($badges)
{
	$group_list = array();
	
	foreach ($badges as $key => $badge)
	{
		if ($badge['group'] > 0)
		{
			$group_list[$badge['group']][] = $key;
		}
	}
	
	return $group_list;
}

/*
* Create a list of badges assigned to a user based on user and group badge assignments.
*
* @param array $user_list An array of default groups keyed by user ID.
* @param array $badge_list An array of badges from the cache system.
* @return array An array of arrays of badge images, keyed by user ID.
*/
function build_user_badge_lists($user_list, $badge_list, $imagelist = array())
{
	global $db;
	
	$user_ids = array_keys ($user_list);
	for ($i = 0, $size = sizeof($user_ids); $i < $size; $i++)
	{
		$user_ids[$i] = intval ($user_ids[$i]);
	}
	
	$badge_groups = build_group_badges($badge_list);

	$sql1 = 'SELECT u.user_id, u.group_id, 0 AS badge_id, 0 as multiple
			 FROM ' . USER_GROUP_TABLE . ' u
			 WHERE u.user_pending = 0
			 	AND ' . $db->sql_in_set('u.user_id', $user_ids);
	
	$sql2 = 'SELECT b.user_id, 0 AS group_id, b.badge_id, b.multiple
			 FROM ' . BADGES_USERS_TABLE . ' b
			 WHERE ' . $db->sql_in_set('b.user_id', $user_ids);
			 
	$result = $db->sql_query("$sql1 UNION $sql2");
	
	$user_badges = array();
	while ($row = $db->sql_fetchrow($result))
	{
		if ($row['group_id'] > 0 && !empty($badge_groups[$row['group_id']]))
		{
			$badge_ids = $badge_groups[$row['group_id']];
			foreach ($badge_ids as $b_id)
			{
				if ($badge_list[$b_id]['group_def_only'] == 0 || $row['group_id'] == $user_list[$row['user_id']])
				{
					$order = $badge_list[$b_id]['order'];
					$user_badges[$row['user_id']][$order] = get_badge_image($badge_list[$b_id], 1, $imagelist);
				}
			}
		}
		else if ($row['badge_id'] > 0)
		{
			$order = $badge_list[$row['badge_id']]['order'];
			$user_badges[$row['user_id']][$order] = get_badge_image($badge_list[$row['badge_id']], $row['multiple'], $imagelist);
		}
	}
	
	$db->sql_freeresult($result);
	
	foreach ($user_badges as $key => $list)
	{
		ksort($list);
		$list = array_values($list);
		$user_badges[$key] = $list;
	}

	return $user_badges;
}

/**
* Accepts a badge description and returns an assembled image
*
* @param array $badge Array describing a badge, taken from cache system.
* @return string A corresponding image string, or the empty string for incomplete data.
*/
function get_badge_image($badge, $multiple = 1, $imagelist = array())
{
	global $config, $phpbb_root_path;
	
	if (empty($badge['img']))
	{
		return '';
	}
	
	$title = ($multiple > 1) ? "{$badge['title']} (x{$multiple})" : $badge['title'];
	
	$src = $badge['img'];
	if (!empty($imagelist) && isset($imagelist[$badge['img']]))
	{
		$set = $imagelist[$badge['img']];
		krsort($set);
		
		foreach ($set as $key => $img)
		{
			if ($multiple >= $key)
			{
				$src = $img['file'];
				break;
			}
		}
	}
	
	$img = '<img src="' . $phpbb_root_path . $config['badge_path'] . '/' . $src . '" alt="' . $badge['alt'] . '" title="' . $title . '" width="' . $badge['width'] . '" height = "' . $badge['height'] . '" />';
	
	return $img;
}

function get_badge_imageset($badgeset, $badgedata, $multiple = 0)
{
	global $config, $phpbb_root_path;
	
	$img = array();
	
	krsort($badgeset);
	$hl_badge = false;
	
	foreach ($badgeset as $key => $badge)
	{
		if (empty($badge['file']))
		{
			continue;
		}
		
		$title = $badgedata['badge_title'] . ' (x' . $key . ')';
		$style = '';
		if ($multiple > 0 && $multiple >= $key && !$hl_badge)
		{
			$hl_badge = true;
		}
		else if ($multiple > 0)
		{
			$style = 'style="opacity: 0.35"';
		}
		
		$img[] = '<img src="' . $phpbb_root_path . $config['badge_path'] . '/' . $badge['file'] . '" alt="' . $badgedata['badge_alt'] . '" title="' . $title . '" width="' . $badge['width'] . '" height = "' . $badge['height'] . '" ' . $style . ' />';
	}
	
	$img = array_reverse($img);
	
	return $img;
}

function badge_imageset()
{
	global $phpbb_root_path, $config;
	
	$imglist = badge_filelist();
	
	$imageset = array();
			
	foreach ($imglist as $path => $img_ary)
	{
		foreach ($img_ary as $img)
		{
			$img_size = getimagesize($phpbb_root_path . $config['badge_path'] . '/' . $path . $img);
			
			if (!$img_size[0] || !$img_size[1] || strlen($img) > 255)
			{
				continue;
			}
			
			$lim = 1;
			$matches = array();
			$imgid = $path . $img;
			
			if (preg_match('/_x([0-9]+)\.\w+$/', $img, $matches))
			{
				$lim = $matches[1];
				$imgid = $path . preg_replace('/_x[0-9]+(\.\w+)$/', '\1', $img);
			}
			
			$imageset[$imgid][$lim]['file'] = $path . $img;
			$imageset[$imgid][$lim]['width'] = $img_size[0];
			$imageset[$imgid][$lim]['height'] = $img_size[1];
		}
	}
	unset($imglist);
	
	foreach ($imageset as $id => $img)
	{
		ksort($imageset[$id]);
	}
	
	return $imageset;
}

/**
* Get physical file listing from badge directory.  We could just use the admin function,
* but we don't want to include that in regular board files.
*/
function badge_filelist($type = 'gif|jpg|jpeg|png')
{
	global $phpbb_root_path, $config;
	
	$matches = array();
	$rootdir = $phpbb_root_path . $config['badge_path'];
	$dir = '';

	// Remove initial / if present
	$rootdir = (substr($rootdir, 0, 1) == '/') ? substr($rootdir, 1) : $rootdir;
	// Add closing / if not present
	$rootdir = ($rootdir && substr($rootdir, -1) != '/') ? $rootdir . '/' : $rootdir;

	// Remove initial / if present
	$dir = (substr($dir, 0, 1) == '/') ? substr($dir, 1) : $dir;
	// Add closing / if not present
	$dir = ($dir && substr($dir, -1) != '/') ? $dir . '/' : $dir;

	if (!is_dir($rootdir . $dir))
	{
		return $matches;
	}

	$dh = @opendir($rootdir . $dir);

	if (!$dh)
	{
		return $matches;
	}

	while (($fname = readdir($dh)) !== false)
	{
		if (is_file("$rootdir$dir$fname"))
		{
			if (filesize("$rootdir$dir$fname") && preg_match('#\.' . $type . '$#i', $fname))
			{
				$matches[$dir][] = $fname;
			}
		}
		else if ($fname[0] != '.' && is_dir("$rootdir$dir$fname"))
		{
			$matches += filelist($rootdir, "$dir$fname", $type);
		}
	}
	closedir($dh);

	return $matches;
}

?>